<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::create('contactable_addresses', function (Blueprint $table) {
            $table->mediumIncrements('id');

            // Relación optimizada (sin morphs por rendimiento)
            $table->unsignedMediumInteger('contactable_id')->index();
            $table->string('contactable_type')->index();

            // Tipo de dirección
            $table->unsignedTinyInteger('type')->index();

            // Ubicación basada en el SAT
            $table->char('c_pais', 3)->charset('ascii')->collation('ascii_general_ci')->nullable()->index();
            $table->unsignedMediumInteger('c_codigo_postal')->nullable()->index();
            $table->char('c_estado', 3)->charset('ascii')->collation('ascii_general_ci')->nullable()->index();
            $table->unsignedTinyInteger('c_localidad')->nullable()->index();
            $table->unsignedSmallInteger('c_municipio')->nullable()->index();
            $table->unsignedMediumInteger('c_colonia')->nullable()->index();

            // Datos de la dirección
            $table->string('direccion')->nullable();
            $table->string('num_ext')->nullable();
            $table->string('num_int')->nullable();
            $table->string('referencia')->nullable();
            $table->decimal('lat', 9, 6)->nullable();
            $table->decimal('lng', 9, 6)->nullable();

            // Preferencia
            $table->unsignedTinyInteger('preference_level')->nullable();

            // Notas o comentarios
            $table->text('notes')->nullable(); // Nuevo campo para comentarios

            // Autoría
            $table->timestamps();

            // Índices
            $table->index(['contactable_type', 'contactable_id']);
            $table->index(['contactable_type', 'contactable_id', 'type']);
            $table->index(['c_municipio', 'c_estado']);

            // Relaciones con catálogos SAT
            $table->foreign('c_pais')->references('c_pais')->on('sat_pais')->onUpdate('restrict')->onDelete('restrict');
            $table->foreign('c_codigo_postal')->references('c_codigo_postal')->on('sat_codigo_postal')->onUpdate('restrict')->onDelete('restrict');
            $table->foreign(['c_estado', 'c_pais'])->references(['c_estado', 'c_pais'])->on('sat_estado')->onUpdate('restrict')->onDelete('restrict');
            $table->foreign(['c_municipio', 'c_estado'])->references(['c_municipio', 'c_estado'])->on('sat_municipio')->onUpdate('restrict')->onDelete('restrict');
            $table->foreign(['c_localidad', 'c_estado'])->references(['c_localidad', 'c_estado'])->on('sat_localidad')->onUpdate('restrict')->onDelete('restrict');
            $table->foreign(['c_colonia', 'c_codigo_postal'])->references(['c_colonia', 'c_codigo_postal'])->on('sat_colonia')->onUpdate('restrict')->onDelete('restrict');
        });

        Schema::create('contactable_items', function (Blueprint $table) {
            $table->mediumIncrements('id');

            // Relación optimizada
            $table->unsignedMediumInteger('contactable_id')->index();
            $table->string('contactable_type')->index();

            // Tipo de medio de contacto (ej. 1=Email, 2=Teléfono, 3=WhatsApp)
            $table->unsignedTinyInteger('type')->index();

            // Dato de contacto (ej. email o teléfono)
            $table->string('data_contact')->index();

            // Preferencia
            $table->unsignedTinyInteger('preference_level')->nullable();

            // Notas o comentarios
            $table->text('notes')->nullable(); // Nuevo campo para comentarios

            // Autoría
            $table->timestamps();

            // Índices
            $table->index(['contactable_type', 'contactable_id']);
            $table->index(['contactable_type', 'contactable_id', 'type']);
        });
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::dropIfExists('contactable_addresses');
        Schema::dropIfExists('contactable_items');
    }
};
